// thinkahead (c) 2010 Greg Toombs
// http://thinkahead.mozdev.org
// $Header: /cvs/thinkahead/src/thinkahead.js,v 1.4 2012/01/27 07:10:41 greg.toombs Exp $

// Lines containing "//debug" are squashed in the release build.

// Code is in this anonymous function to prevent variables from being public,
// decreasing namespace pollution and improving compression.
// See http://compressorrater.thruhere.net/ .
(function() {
	// Components.utils.import breaks with some dumb parsers.
	Components.utils['import']('resource://gre/modules/XPCOMUtils.jsm');
	
	// All vars at this scope are private and subject to renaming.
	var
	conserv = Cc['@mozilla.org/consoleservice;1'  ].getService(Ci.nsIConsoleService),	//debug
	obsserv = Cc['@mozilla.org/observer-service;1'].getService(Ci.nsIObserverService),
	topics = [
		'http-on-modify-request',
		'http-on-examine-response',
		'http-on-examine-cached-response'],
	
	log = function(message) {				//debug
		conserv.logStringMessage(message);	//debug
	},										//debug
	
	Observer = {
		// Names at this scope are preserved, which is good because we're
		// implementing an interface.
		QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver]),
		
		observe: function(aSubject, aTopic, aData) {
			// This try block often throws up on interface and null errors.
			// Who really cares?
			try {
				var channel = aSubject.QueryInterface(Ci.nsIHttpChannel),
					msg = 'ThinkAhead match on event ' + aTopic + ':\n' +	//debug
						'\tURI: ' + channel.URI.spec + '\n',				//debug
					cbs = channel.notificationCallbacks;
				if (!cbs)
					cbs = channel.loadGroup.notificationCallbacks;
				
				var doc = cbs.getInterface(Ci.nsIDOMWindow).document,
					matches = doc.TA_Matches;
			} catch(e) {
				return;
			}
			
			if ('object' == typeof matches) {
				var reset = doc.getElementById('TA_Reset');
				if (reset) {
					matches = 0;
					reset.parentNode.removeChild(reset);
					log('ThinkAhead: Resetting ' + doc.documentURI);	//debug
				}
			}
			if ('object' != typeof matches) {
				matches = [];
				var telms = doc.evaluate(
					"//script[@type='application/json' and contains(.,'thinkahead_ver')]",
					doc, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
				for (t = 0; t < telms.snapshotLength; t++) {
					var ta = JSON.parse(telms.snapshotItem(t).innerHTML);
					if (ta.thinkahead_ver == '1.04') {
						log('Parsing ThinkAhead block in ' + doc.documentURI);	//debug
						var tmatches = ta.matches;
						for (m in tmatches) {
							var rexes = tmatches[m].regexurls;
							for (u in rexes)
								rexes[u] = new RegExp(rexes[u]);
						}
						matches = matches.concat(tmatches);
					}
					else											//debug
						log('Incompatible ThinkAhead version.\n');	//debug
				}
				doc.TA_Matches = matches;
			}
			
			for (m in matches) {
				var match = matches[m],
					pass = false,
					urls = match.urls;
				for (u in urls)
					if (urls[u] == channel.URI.spec) {
						msg += '\tMatched to url: ' + urls[u] + '\n';	//debug
						pass = true;
						break;
					}
				if (!pass) {
					urls = match.regexurls;
					for (u in urls)
						if (urls[u].test(channel.URI.spec)) {
							msg += '\tMatch to regexurl: ' +			//debug
								urls[u].source + '\n';					//debug
							pass = true;
							break;
						}
				}
				if (pass) {
					if (aTopic == topics[0])
						var heads = match.request,
							headfun = channel.setRequestHeader;
					else
						var heads = match.response,
							headfun = channel.setResponseHeader;
					for (h in heads)
					{														//debug
						msg += '\tSetting ' + h + '=' + heads[h] + '\n';	//debug
						headfun(h, heads[h], false);
					}														//debug
					log(msg);												//debug
					msg = '';												//debug
				}
			}
		}
	},
	
	load = function(event, obsop) {
		window.addEventListener(event, function() {
			for (i in topics)
				obsserv[obsop + 'Observer'](Observer, topics[i], false);
			}, false);
	}
	
	load(  'load',    'add');
	load('unload', 'remove');
})();

